@@ -16,8 +16,37 @@ pub use ser::Serializer;
 type Result<T> = std::result::Result<T, Error>;
 
 fn static_str_to_js(s: &'static str) -> JsString {
+    use std::cell::RefCell;
+    use std::collections::HashMap;
+
+    #[derive(Default)]
+    struct PtrHasher {
+        addr: usize,
+    }
+
+    impl std::hash::Hasher for PtrHasher {
+        fn write(&mut self, _bytes: &[u8]) {
+            unreachable!();
+        }
+
+        fn write_usize(&mut self, addr_or_len: usize) {
+            if self.addr == 0 {
+                self.addr = addr_or_len;
+            }
+        }
+
+        fn finish(&self) -> u64 {
+            self.addr as _
+        }
+    }
+
+    type PtrBuildHasher = std::hash::BuildHasherDefault<PtrHasher>;
+
     thread_local! {
-        static CACHE: std::cell::RefCell<fnv::FnvHashMap<&'static str, JsString>> = Default::default();
+        // Since we're mainly optimising for converting the exact same string literal over and over again,
+        // which will always have the same pointer, we can speed things up by indexing by the string's pointer
+        // instead of its value.
+        static CACHE: RefCell<HashMap<*const str, JsString, PtrBuildHasher>> = Default::default();
     }
     CACHE.with(|cache| {
         cache
