@@ -4,7 +4,7 @@ use serde::de::DeserializeOwned;
 use serde::ser::Error as SerError;
 use serde::{Deserialize, Serialize};
 use serde_wasm_bindgen::{from_value, to_value, Error, Serializer};
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::fmt::Debug;
 use std::hash::Hash;
 use wasm_bindgen::{JsCast, JsValue};
@@ -69,6 +69,16 @@ where
     test_via_json_with_config(value, Serializer::new());
 }
 
+fn test_via_round_trip<T>(value: T, serializer: Serializer)
+where
+    T: Serialize + DeserializeOwned + PartialEq + Debug + Clone,
+{
+    let original = value.clone();
+    let serialized = value.serialize(&serializer).unwrap();
+    let round_trip = from_value(serialized).unwrap();
+    assert_eq!(original, round_trip);
+}
+
 macro_rules! test_unsigned {
     ($ty:ident) => {{
         test_primitive::<$ty>(42 as _);
@@ -388,7 +398,7 @@ fn enums() {
         ExternallyTagged
     }
 
-    #[derive(Debug, PartialEq, Serialize, Deserialize)]
+    #[derive(Debug, PartialEq, Serialize, Deserialize, Clone)]
     #[serde(tag = "tag")]
     enum InternallyTagged<A, B>
     where
@@ -425,6 +435,14 @@ fn enums() {
         Serializer::new().serialize_maps_as_objects(true),
     );
 
+    test_via_round_trip(
+        InternallyTagged::Struct {
+            a: 10_u64,
+            b: -10_i64,
+        },
+        Serializer::new().serialize_large_number_types_as_bigints(true),
+    );
+
     test_enum! {
         #[serde(tag = "tag", content = "content")]
         AdjacentlyTagged
@@ -541,6 +559,23 @@ fn maps_objects_string_key() {
     test_via_json_with_config(src, Serializer::new().serialize_maps_as_objects(true));
 }
 
+#[wasm_bindgen_test]
+fn serialize_json_compatible() {
+    #[derive(Debug, Serialize, Deserialize, PartialEq, Eq)]
+    struct Struct {
+        a: HashMap<String, ()>,
+        b: Option<i32>,
+    }
+    let x = Struct {
+        a: hashmap! {
+            "foo".to_string() => (),
+            "bar".to_string() => (),
+        },
+        b: None,
+    };
+    test_via_json_with_config(x, Serializer::json_compatible());
+}
+
 #[wasm_bindgen_test]
 fn maps_objects_object_key() {
     #[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Hash)]
